from ..methods import Methods
from typing import (
    Optional,
    Union,
    List,
    Literal
)
from ...core.async_sync import *
from ..filters import Filter

class Client(object):

    def __init__(
        self,
        session:Optional[str]=None,
        auth:Optional[str]=None,
        private:Optional[str]=None,
        platform:str="web",
        api_version:int=6,
        proxy:Optional[str]=None,
        time_out=10,
        show_progress_bar:bool=True
    ):
        
        self.session = session
        self.platform = platform
        self.apiVersion = api_version
        self.proxy = proxy
        self.timeOut = time_out
        
        if(session):
            from ..sessions import Sessions
            self.sessions = Sessions(self)

            if(self.sessions.cheackSessionExists()):
                self.sessionData = self.sessions.loadSessionData()
            else:
                self.sessionData = self.sessions.createSession()
        else:
            from ..utils import Utils
            self.sessionData = {
                "auth": auth,
                "private_key": Utils.privateParse(private=private) if private else None
            }

        self.methods = Methods(
            sessionData=self.sessionData,
            platform=platform,
            apiVersion=api_version,
            proxy=proxy,
            timeOut=time_out,
            showProgressBar=show_progress_bar
        )

    # propertys
    
    @property
    def auth(self):
        return self.sessionData["auth"]
    
    @property
    def private_key(self):
        return self.sessionData["private_key"]

    # Authentication methods
    
    @async_to_sync
    async def send_code(self, phone_number:str, pass_key:Optional[str] = None) -> dict:
        return await self.methods.sendCode(phoneNumber=phone_number, passKey=pass_key)
    
    @async_to_sync
    async def sign_in(self, phone_number:str, phone_code_hash:str, phone_code:str) -> dict:
        return await self.methods.signIn(phoneNumber=phone_number, phoneCodeHash=phone_code_hash, phoneCode=phone_code)
    
    @async_to_sync
    async def register_device(self, device_model:str) -> dict:
        return await self.methods.registerDevice(deviceModel=device_model)
    
    @async_to_sync
    async def logout(self) -> dict:
        return await self.methods.logout()
    
    # Chats methods

    @async_to_sync
    async def get_chats(self, start_id:Optional[str] = None) -> dict:
        return await self.methods.getChats(startId=start_id)
    
    @async_to_sync
    async def get_object_by_username(self,user_name: str) -> dict:
        return await self.methods.getObjectByUsername(user_name)

    @async_to_sync
    async def get_top_users(self) -> dict:
        return await self.methods.getTopChatUsers()
    
    @async_to_sync
    async def remove_from_top_users(self, object_guid:str) -> dict:
        return await self.methods.removeFromTopChatUsers(objectGuid=object_guid)
    
    @async_to_sync
    async def get_chat_ads(self) -> dict:
        return await self.methods.getChatAds()
    
    @async_to_sync
    async def get_chats_updates(self) -> dict:
        return await self.methods.getChatsUpdates()
    
    @async_to_sync
    async def join_chat(self, guid_or_link:str) -> dict:
        return await self.methods.joinChat(guidOrLink=guid_or_link)

    @async_to_sync
    async def action_on_join_request(self, object_guid: str, user_guid: str, action: Literal["Accept", "Reject"]):
        return await self.methods.actionOnJoinRequest(objectGuid=object_guid, userGuid=user_guid, action=action)
    
    @async_to_sync
    async def getJoinRequests(self, object_guid: str):
        return await self.methods.getJoinRequests(objectGuid=object_guid)
    
    @async_to_sync
    async def leave_chat(self, object_guid:str) -> dict:
        return await self.methods.leaveChat(objectGuid=object_guid)
    
    @async_to_sync
    async def remove_chat(self, object_guid:str) -> dict:
        return await self.methods.removeChat(objectGuid=object_guid)
    
    @async_to_sync
    async def get_chat_info(self, object_guid:str) -> dict:
        return await self.methods.getChatInfo(objectGuid=object_guid)
    
    @async_to_sync
    async def get_chat_info_by_username(self, username:str) -> dict:
        return await self.methods.getChatInfoByUsername(username=username)

    @async_to_sync
    async def get_link(self, object_guid:str) -> dict:
        return await self.methods.getChatLink(objectGuid=object_guid)
    
    @async_to_sync
    async def set_link(self, object_guid:str) -> dict:
        return await self.methods.setChatLink(objectGuid=object_guid)
    
    @async_to_sync
    async def set_admin(self, object_guid:str, member_guid:str, access_list:Optional[list], custom_title:Optional[str] = None) -> dict:
        return await self.methods.setChatAdmin(objectGuid=object_guid, memberGuid=member_guid, accessList=access_list, customTitle=custom_title, action="SetAdmin")
    
    @async_to_sync
    async def unset_admin(self, object_guid:str, member_guid:str) -> dict:
        return await self.methods.setChatAdmin(objectGuid=object_guid, memberGuid=member_guid, accessList=None, customTitle=None, action="UnsetAdmin")
    
    @async_to_sync
    async def add_member(self, object_guid:str, member_guids:Union[list,str]) -> dict:
        if isinstance(member_guids,str):
            member_guids = [member_guids]
        return await self.methods.addChatMember(objectGuid=object_guid, memberGuids=member_guids)

    @async_to_sync
    async def ban_member(self, object_guid:str, member_guid:str) -> dict:
        return await self.methods.banChatMember(objectGuid=object_guid, memberGuid=member_guid, action="Set")
    
    @async_to_sync
    async def unban_member(self, object_guid:str, member_guid:str) -> dict:
        return await self.methods.banChatMember(objectGuid=object_guid, memberGuid=member_guid, action="Unset")
    
    @async_to_sync
    async def get_banned_members(self, object_guid:str, start_id:Optional[str] = None) -> dict:
        return await self.methods.getBannedChatMembers(objectGuid=object_guid, startId=start_id)

    @async_to_sync
    async def get_all_members(self, object_guid:str, search_text:Optional[str] = None, start_id:Optional[str] = None, just_get_guids:bool=False) -> Union[dict,list]:
        return await self.methods.getChatAllMembers(objectGuid=object_guid, searchText=search_text, startId=start_id, justGetGuids=just_get_guids)
    
    @async_to_sync
    async def get_admin_members(self, object_guid:str, start_id:Optional[str] = None, just_get_guids:bool=False) -> Union[dict,list]:
        return await self.methods.getChatAdminMembers(objectGuid=object_guid, startId=start_id, justGetGuids=just_get_guids)
    
    @async_to_sync
    async def user_is_admin(self, object_guid: str, user_guid: str):
        return await self.methods.userIsAdmin(objectGuid=object_guid,userGuid=user_guid)
    
    @async_to_sync
    async def get_admin_access_list(self, object_guid:str, member_guid:str) -> dict:
        return await self.methods.getChatAdminAccessList(objectGuid=object_guid, memberGuid=member_guid)
    
    @async_to_sync
    async def get_chat_preview(self, link:str) -> dict:
        return await self.methods.chatPreviewByJoinLink(link=link)
    
    @async_to_sync
    async def create_voice_chat(self, object_guid:str) -> dict:
        return await self.methods.createChatVoiceChat(objectGuid=object_guid)
    
    @async_to_sync
    async def join_voice_chat(self, object_guid:str, my_guid:str, voice_chat_id:str,sdp_offer_data:str) -> dict:
        return await self.methods.joinVoiceChat(objectGuid=object_guid, myGuid=my_guid, voiceChatId=voice_chat_id,sdp_offer_data=sdp_offer_data)
    
    @async_to_sync
    async def set_voice_chat_setting(self, object_guid:str, voice_chat_id:str, title:Optional[str] = None, join_mute:Optional[bool]=None) -> dict:
        return await self.methods.setChatVoiceChatSetting(objectGuid=object_guid, voideChatId=voice_chat_id, title=title, joinMuted=join_mute)
    
    @async_to_sync
    async def get_voice_chat_updates(self, object_guid:str, voice_chat_id:str) -> dict:
        return await self.methods.getChatVoiceChatUpdates(objectGuid=object_guid, voideChatId=voice_chat_id)
    
    @async_to_sync
    async def get_voice_chat_participants(self, object_guid:str, voice_chat_id:str) -> dict:
        return await self.methods.getChatVoiceChatParticipants(objectGuid=object_guid, voideChatId=voice_chat_id)
    
    @async_to_sync
    async def set_voice_chat_state(self, object_guid:str, voice_chat_id:str, activity:str,participantObjectGuid:str) -> dict:
        return await self.methods.setChatVoiceChatState(objectGuid=object_guid, voideChatId=voice_chat_id, activity=activity,participantObjectGuid=participantObjectGuid)
    
    @async_to_sync
    async def send_voice_chat_activity(self, object_guid:str, voice_chat_id:str, activity:str, participant_object_guid:str) -> dict:
        return await self.methods.sendChatVoiceChatActivity(objectGuid=object_guid, voideChatId=voice_chat_id, activity=activity, participantObjectGuid=participant_object_guid)    
    
    @async_to_sync
    async def leave_voice_chat(self, object_guid:str, voice_chat_id:str) -> dict:
        return await self.methods.leaveChatVoiceChat(objectGuid=object_guid, voideChatId=voice_chat_id)
    
    @async_to_sync
    async def discard_voice_chat(self, object_guid:str, voice_chat_id:str) -> dict:
        return await self.methods.discardChatVoiceChat(objectGuid=object_guid, voideChatId=voice_chat_id)
    
    @async_to_sync
    async def pin_chat(self, object_guid:str) -> dict:
        return await self.methods.setActionChat(objectGuid=object_guid, action="Pin")
    
    @async_to_sync
    async def unpin_chat(self, object_guid:str) -> dict:
        return await self.methods.setActionChat(objectGuid=object_guid, action="Unpin")
    
    @async_to_sync
    async def mute_chat(self, object_guid:str) -> dict:
        return await self.methods.setActionChat(objectGuid=object_guid, action="Mute")
    
    @async_to_sync
    async def unmute_chat(self, object_guid:str) -> dict:
        return await self.methods.setActionChat(objectGuid=object_guid, action="Unmute")
    
    @async_to_sync
    async def seen_chats(self, seen_list:dict) -> dict:
        """
        ```python

        seen_list : dict = {"object_guid": "message_id", "object_guid": "message_id", ...}

        ```
        """
        return await self.methods.seenChats(seenList=seen_list)
    
    @async_to_sync
    async def send_chat_activity(self, object_guid:str, activity:str) -> dict:
        return await self.methods.sendChatActivity(objectGuid=object_guid, activity=activity)
    
    @async_to_sync
    async def search_chat_messages(self, object_guid:str, search_text:str) -> dict:
        return await self.methods.searchChatMessages(objectGuid=object_guid, searchText=search_text)
    
    @async_to_sync
    async def upload_avatar(self, object_guid:str, main_file:str, thumbnail_file:Optional[str] = None) -> Optional[dict]:
        return await self.methods.uploadAvatar(objectGuid=object_guid, mainFile=main_file, thumbnailFile=thumbnail_file)
    
    @async_to_sync
    async def getAvatars(self, object_guid:str) -> dict:
        return await self.methods.getAvatars(objectGuid=object_guid)
    
    @async_to_sync
    async def delete_avatar(self, object_guid:str, avatar_id:str) -> dict:
        return await self.methods.deleteAvatar(objectGuid=object_guid, avatarId=avatar_id)
    
    @async_to_sync
    async def delete_history(self, object_guid:str, last_message_id:str) -> dict:
        return await self.methods.deleteChatHistory(objectGuid=object_guid, lastMessageId=last_message_id)
    
    @async_to_sync
    async def delete_user_chat(self, user_guid:str, last_deleted_message_id:str) -> dict:
        return await self.methods.deleteUserChat(userGuid=user_guid, lastDeletedMessageId=last_deleted_message_id)
    
    @async_to_sync
    async def get_pending_owner(self, object_guid:str) -> dict:
        return await self.methods.getPendingObjectOwner(objectGuid=object_guid)
    
    @async_to_sync
    async def request_change_owner(self, object_guid:str, member_guid:str) -> dict:
        return await self.methods.requestChangeObjectOwner(objectGuid=object_guid, memberGuid=member_guid)
    
    @async_to_sync
    async def accept_request_owner(self, object_guid:str) -> dict:
        return await self.methods.replyRequestObjectOwner(objectGuid=object_guid, action="Accept")
    
    @async_to_sync
    async def reject_request_owner(self, object_guid:str) -> dict:
        return await self.methods.replyRequestObjectOwner(objectGuid=object_guid, action="Reject")
    
    @async_to_sync
    async def cancel_change_owner(self, object_guid:str) -> dict:
        return await self.methods.cancelChangeObjectOwner(objectGuid=object_guid)
    
    @async_to_sync
    async def get_chat_reaction(self, object_guid:str, min_id:str, max_id:str) -> dict:
        return await self.methods.getChatReaction(objectGuid=object_guid, minId=min_id, maxId=max_id)
    
    @async_to_sync
    async def report_chat(self, object_guid:str, description:str) -> dict:
        return await self.methods.reportObject(objectGuid=object_guid, description=description)
    
    @async_to_sync
    async def set_chat_use_time(self, object_guid:str, time:int) -> dict:
        return await self.methods.setChatUseTime(objectGuid=object_guid, time=time)
    
    # User methods
    
    @async_to_sync
    async def block_user(self, object_guid:str) -> dict:
        return await self.methods.setBlockUser(objectGuid=object_guid, action="Block")
    
    @async_to_sync
    async def unblock_user(self, object_guid:str) -> dict:
        return await self.methods.setBlockUser(objectGuid=object_guid, action="Unblock")
    
    @async_to_sync
    async def check_user_username(self, username:str) -> dict:
        return await self.methods.checkUserUsername(username=username)
    
    # Group methods
    
    @async_to_sync
    async def add_group(self, title:str, member_guids:list) -> dict:
        return await self.methods.addGroup(title=title, memberGuids=member_guids)
    
    @async_to_sync
    async def get_group_default_access(self, object_guid:str) -> dict:
        return await self.methods.getGroupDefaultAccess(objectGuid=object_guid)
    
    @async_to_sync
    async def set_group_default_access(self, object_guid:str, access_list:list=[]) -> dict:
        return await self.methods.setChatDefaultAccess(objectGuid=object_guid, accessList=access_list)

    @async_to_sync
    async def get_group_mention_list(self, object_guid:str, search_mention:str) -> dict:
        return await self.methods.getGroupMentionList(objectGuid=object_guid, searchMention=search_mention)
    
    @async_to_sync
    async def edit_group_info(self, object_guid:str, title:Optional[str] = None, description:Optional[str] = None, slow_mode:Optional[int]=None, event_messages:Optional[bool]=None, chat_history_for_new_members:Optional[bool]=None, reaction_type:Optional[str] = None, selected_reactions:Optional[list]=None) -> dict:
        return await self.methods.editGroupInfo(objectGuid=object_guid, title=title, description=description, slowMode=slow_mode, eventMessages=event_messages, chatHistoryForNewMembers=chat_history_for_new_members, reactionType=reaction_type, selectedReactions=selected_reactions)
    
    # Channels methods
    
    @async_to_sync
    async def add_channel(self, title:str, description:Optional[str] = None, member_guids:Optional[list]=None, private:bool=False) -> dict:
        return await self.methods.addChannel(title=title, description=description, memberGuids=member_guids, private=private)

    @async_to_sync
    async def edit_channel_info(self, object_guid:str, title:Optional[str] = None, description:Optional[str] = None, username:Optional[str] = None, private:Optional[bool]=None, sign_message:Optional[bool]=None, reaction_type:Optional[str] = None, selected_reactions:Optional[list]=None) -> dict:
        return await self.methods.editChannelInfo(objectGuid=object_guid, title=title, description=description, username=username, private=private, signMessages=sign_message, reactionType=reaction_type, selectedReactions=selected_reactions)
    
    @async_to_sync
    async def check_channel_username(self, username:str) -> dict:
        return await self.methods.checkChannelUsername(username=username)
    
    @async_to_sync
    async def get_channel_seen_count(self, object_guid:str, min_id:str, max_id:str) -> dict:
        return await self.methods.getChannelSeenCount(objectGuid=object_guid, minId=min_id, maxId=max_id)
    
    # Message methods
    
    @async_to_sync
    async def send_text(self, object_guid:str, text:str, message_id:Optional[str] = None) -> dict:
        return await self.methods.sendText(objectGuid=object_guid, text=text, messageId=message_id)
    
    @async_to_sync
    async def send_file(self, object_guid:str, file:str, message_id:Optional[str] = None, text:Optional[str] = None, file_name:Optional[str] = None) -> Optional[dict]:
        return await self.methods.sendFile(objectGuid=object_guid, file=file, text=text, messageId=message_id, fileName=file_name)
    
    @async_to_sync
    async def send_image(self, object_guid:str, file:str, message_id:Optional[str] = None, text:Optional[str] = None, is_spoil:bool=False, thumbnail:Optional[str] = None, file_name:Optional[str] = None) -> Optional[dict]:
        return await self.methods.sendImage(objectGuid=object_guid, file=file, text=text, messageId=message_id, isSpoil=is_spoil, thumbInline=thumbnail, fileName=file_name)
    
    @async_to_sync
    async def send_video(self, object_guid:str, file:str, message_id:Optional[str] = None, text:Optional[str] = None, is_spoil:bool=False, thumbnail:Optional[str] = None, file_name:Optional[str] = None) -> Optional[dict]:
        return await self.methods.sendVideo(objectGuid=object_guid, file=file, text=text, messageId=message_id, isSpoil=is_spoil, thumbInline=thumbnail, fileName=file_name)
    
    @async_to_sync
    async def send_video_message(self, object_guid:str, file:str, message_id:Optional[str] = None, text:Optional[str] = None, thumbnail:Optional[str] = None, file_name:Optional[str] = None) -> Optional[dict]:
        return await self.methods.sendVideoMessage(objectGuid=object_guid, file=file, text=text, messageId=message_id, thumbInline=thumbnail, fileName=file_name)
    
    @async_to_sync
    async def send_gif(self, object_guid:str, file:str, message_id:Optional[str] = None, text:Optional[str] = None, thumbnail:Optional[str] = None, file_name:Optional[str] = None) -> Optional[dict]:
        return await self.methods.sendGif(objectGuid=object_guid, file=file, text=text, messageId=message_id, thumbInline=thumbnail, fileName=file_name)
    
    @async_to_sync
    async def send_music(self, object_guid:str, file:str, message_id:Optional[str] = None, text:Optional[str] = None, file_name:Optional[str] = None, performer:Optional[str] = None) -> Optional[dict]:
        return await self.methods.sendMusic(objectGuid=object_guid, file=file, text=text, messageId=message_id, fileName=file_name, performer=performer)
    
    @async_to_sync
    async def send_voice(self, object_guid:str, file:str, message_id:Optional[str] = None, text:Optional[str] = None, file_name:Optional[str] = None, time:int=0) -> Optional[dict]:
        return await self.methods.sendVoice(objectGuid=object_guid, file=file, text=text, messageId=message_id, fileName=file_name, time=time)
    
    @async_to_sync
    async def send_location(self, object_guid:str, latitude:int, longitude:int, message_id:Optional[str] = None) -> dict:
        return await self.methods.sendLocation(objectGuid=object_guid, latitude=latitude, longitude=longitude, messageId=message_id)
    
    @async_to_sync
    async def send_message_api_call(self, objectGuid:str, text:str, message_id:str, button_id:str) -> dict:
        return await self.methods.sendMessageAPICall(objectGuid=objectGuid, text=text, messageId=message_id, buttonId=button_id)
    
    @async_to_sync
    async def reaction_message(self, object_guid: str, message_id: str, reaction: Union[int,str]) -> dict:
        return await self.methods.actionOnMessageReaction(objectGuid=object_guid, messageId=message_id, reactionId=reaction, action="Add")
    
    @async_to_sync
    async def unreaction_message(self, object_guid: str, message_id: str, reaction: Union[int,str]) -> dict:
        return await self.methods.actionOnMessageReaction(objectGuid=object_guid, messageId=message_id, reactionId=reaction, action="Remove")
    
    @async_to_sync
    async def pin_message(self, object_guid:str, message_id:str) -> dict:
        return await self.methods.setPinMessage(objectGuid=object_guid, messageId=message_id, action="Pin")
    
    @async_to_sync
    async def unpin_message(self, object_guid:str, message_id:str) -> dict:
        return await self.methods.setPinMessage(objectGuid=object_guid, messageId=message_id, action="Unpin")
    
    @async_to_sync
    async def resend_message(self, object_guid:Optional[str] = None, message_id:Optional[str] = None, to_object_guid:Optional[str] = None, reply_to_message_id:Optional[str] = None, text:Optional[str] = None, file_inline:Optional[dict]=None) -> dict:
        return await self.methods.resendMessage(objectGuid=object_guid, messageId=message_id, toObjectGuid=to_object_guid, replyToMessageId=reply_to_message_id, text=text, fileInline=file_inline)
    
    @async_to_sync
    async def forward_messages(self, object_guid:str, message_ids:list, to_object_guid:str) -> dict:
        return await self.methods.forwardMessages(objectGuid=object_guid, messageIds=message_ids, toObjectGuid=to_object_guid)
    
    @async_to_sync
    async def edit_message(self, object_guid, text, message_id=None) -> dict:
        return await self.methods.editMessage(object_guid, text=text, messageId=message_id)
    
    @async_to_sync
    async def delete_messages(self, object_guid:str, message_ids:list, delete_for_all:bool=True) -> dict:
        return await self.methods.deleteMessages(objectGuid=object_guid, messageIds=message_ids, deleteForAll=delete_for_all)

    @async_to_sync
    async def auto_delete(self, object_guid: str, message_id: str, time: int, delete_for_all: bool = True):
        return await self.methods.autoDelete(objectGuid=object_guid,messageId=message_id,time=time,deleteForAll=delete_for_all)
    
    @async_to_sync
    async def seen_messages(self, object_guid:str, min_id:str, max_id:str) -> dict:
        return await self.methods.seenChatMessages(objectGuid=object_guid, minId=min_id, maxId=max_id)
    
    @async_to_sync
    async def get_messages_interval(self, object_guid:str, middle_message_id:str) -> dict:
        return await self.methods.getMessagesInterval(objectGuid=object_guid, middleMessageId=middle_message_id)
    
    @async_to_sync
    async def get_messages(self, object_guid:str, max_message_id:Optional[str] = None, filter_type:Optional[str] = None, limit:int=50) -> dict:
        return await self.methods.getMessages(objectGuid=object_guid, maxId=max_message_id, filterType=filter_type, limit=limit)
    
    @async_to_sync
    async def get_last_message(self, object_guid:str) -> dict:
        return (await self.methods.getChatInfo(objectGuid=object_guid))["chat"]["last_message"]
    
    @async_to_sync
    async def get_last_message_id(self, object_guid:str) -> str:
        return (await self.methods.getChatInfo(objectGuid=object_guid))["chat"]["last_message_id"]
    
    @async_to_sync
    async def get_messages_updates(self, object_guid:str) -> dict:
        return await self.methods.getMessagesUpdates(objectGuid=object_guid)
    
    @async_to_sync
    async def get_messages_by_id(self, object_guid:str, message_ids:list) -> dict:
        return await self.methods.getMessagesById(objectGuid=object_guid, messageIds=message_ids)
    
    @async_to_sync
    async def get_message_share_url(self, object_guid:str, message_id:str) -> dict:
        return await self.methods.getMessageShareUrl(objectGuid=object_guid, messageId=message_id)
    
    @async_to_sync
    async def click_message_url(self, object_guid:str, message_id:str, link_url:str) -> dict:
        return await self.methods.clickMessageUrl(objectGuid=object_guid, messageId=message_id, linkUrl=link_url)
    
    @async_to_sync
    async def request_send_file(self, file_name:str, mime:str, size:int) -> dict:
        return await self.methods.requestSendFile(fileName=file_name, mime=mime, size=size)
    
    # Contact methods

    @async_to_sync
    async def send_contact(self, object_guid:str, first_name:str, last_name:str, phone_number:str, user_guid:str, message_id:Optional[str] = None) -> dict:
        return await self.methods.sendContact(objectGuid=object_guid, firstName=first_name, lastName=last_name, phoneNumber=phone_number, userGuid=user_guid, messageId=message_id)
    
    @async_to_sync
    async def get_contacts(self, start_id:Optional[str] = None) -> dict:
        return await self.methods.getContacts(startId=start_id)
    
    @async_to_sync
    async def get_contacts_last_online(self, user_guids:list) -> dict:
        return await self.methods.getContactsLastOnline(userGuids=user_guids)
    
    @async_to_sync
    async def add_address_book(self, phone:str, first_name:str, last_name:str) -> dict:
        return await self.methods.addAddressBook(phone=phone, firstName=first_name, lastName=last_name)
    
    @async_to_sync
    async def delete_contact(self, object_guid:str) -> dict:
        return await self.methods.deleteContact(objectGuid=object_guid)
    
    @async_to_sync
    async def get_contacts_updates(self) -> dict:
        return await self.methods.getContactsUpdates()
    
    # Sticker methods

    @async_to_sync
    async def send_sticker(self, object_guid:str, emoji:Optional[str] = None, message_id:Optional[str] = None, sticker_data:Optional[str]=None) -> dict:
        return await self.methods.sendSticker(objectGuid=object_guid, emoji=emoji, messageId=message_id, stickerData=sticker_data)
    
    @async_to_sync
    async def get_my_sticker_sets(self) -> dict:
        return await self.methods.getMyStickerSets()
    
    @async_to_sync
    async def get_trend_sticker_sets(self, start_id:Optional[str] = None) -> dict:
        return await self.methods.getTrendStickerSets(startId=start_id)
    
    @async_to_sync
    async def search_stickers(self, search_text:str, start_id:Optional[str] = None) -> dict:
        return await self.methods.searchStickers(searchText=search_text, startId=start_id)
    
    @async_to_sync
    async def add_sticker(self, sticker_set_id:str) -> dict:
        return await self.methods.actionOnStickerSet(stickerSetId=sticker_set_id, action="Add")
    
    @async_to_sync
    async def remove_sticker(self, sticker_set_id:str) -> dict:
        return await self.methods.actionOnStickerSet(stickerSetId=sticker_set_id, action="Remove")
    
    @async_to_sync
    async def get_stickers_by_emoji(self, emoji:str) -> dict:
        return await self.methods.getStickersByEmoji(emoji=emoji)
    
    @async_to_sync
    async def get_stickers_by_set_ids(self, sticker_set_ids:list) -> dict:
        return await self.methods.getStickersBySetIDs(stickerSetIds=sticker_set_ids)
    
    # Gif methods

    @async_to_sync
    async def get_my_gif_set(self) -> dict:
        return await self.methods.getMyGifSet()
    
    @async_to_sync
    async def add_gif(self, object_guid:str, message_id:str) -> dict:
        return await self.methods.addToMyGifSet(objectGuid=object_guid, messageId=message_id)
    
    @async_to_sync
    async def remove_gif(self, file_id:str) -> dict:
        return await self.methods.removeFromMyGifSet(fileId=file_id)
    
    # Poll methods

    @async_to_sync
    async def send_poll(self, object_guid:str, question:str, options:list, message_id:Optional[str] = None, multiple_answers:Optional[bool]=None, anonymous:Optional[bool]=None, quiz:Optional[bool]=None) -> dict:
        return await self.methods.sendPoll(objectGuid=object_guid, question=question, options=options, messageId=message_id, multipleAnswers=multiple_answers, anonymous=anonymous, quiz=quiz)
    
    @async_to_sync
    async def vote_poll(self, poll_id:str, selection_index:int) -> dict:
        return await self.methods.votePoll(pollId=poll_id, selectionIndex=selection_index)
    
    @async_to_sync
    async def get_poll_status(self, poll_id:str) -> dict:
        return await self.methods.getPollStatus(pollId=poll_id)
    
    @async_to_sync
    async def get_poll_option_voters(self, poll_id:str, selection_index:int, start_id:Optional[str] = None) -> dict:
        return await self.methods.getPollOptionVoters(pollId=poll_id, selectionIndex=selection_index, startId=start_id)
    
    # Live methods
    
    @async_to_sync
    async def send_live(self, object_guid:str, thumb_inline:str) -> dict:
        return await self.methods.sendLive(objectGuid=object_guid, thumbInline=thumb_inline)
    
    @async_to_sync
    async def add_live_comment(self, access_token:str, live_id:str, text:str) -> dict:
        return await self.methods.addLiveComment(accessToken=access_token, liveId=live_id, text=text)
    
    @async_to_sync
    async def get_live_status(self, access_token:str, live_id:str) -> dict:
        return await self.methods.getLiveStatus(accessToken=access_token, liveId=live_id)
    
    @async_to_sync
    async def getLiveComments(self, access_token:str, live_id:str) -> dict:
        return await self.methods.getLiveComments(accessToken=access_token, liveId=live_id)
    
    @async_to_sync
    async def getLivePlayUrl(self, access_token:str, live_id:str) -> dict:
        return await self.methods.getLivePlayUrl(accessToken=access_token, liveId=live_id)
    
    # Call methods

    @async_to_sync
    async def requestCall(self, object_guid:str, call_type:str) -> dict:
        return await self.methods.requestCall(objectGuid=object_guid, callType=call_type)
    
    @async_to_sync
    async def discard_call(self, call_id:str, duration:int, reason:str) -> dict:
        return await self.methods.discardCall(callId=call_id, duration=duration, reason=reason)
    
    # Setting methods
    
    @async_to_sync
    async def set_setting(self, show_my_last_online:Optional[bool]=None, show_my_phone_number:Optional[bool]=None, show_my_profile_photo:Optional[bool]=None, link_forward_message:Optional[bool]=None, can_join_chat_by:Optional[bool]=None) -> dict:
        return await self.methods.setSetting(showMyLastOnline=show_my_last_online, showMyPhoneNumber=show_my_phone_number, showMyProfilePhoto=show_my_profile_photo, linkForwardMessage=link_forward_message, canJoinChatBy=can_join_chat_by)
    
    @async_to_sync
    async def add_folder(self, folder_name:str, folder_id:str, exclude_chat_ids:list, exclude_chat_types:list, include_chat_ids:list, include_chat_types:list) -> dict:
        return await self.methods.addFolder(folderName=folder_name, folderId=folder_id, excludeChatIds=exclude_chat_ids, excludeChatTypes=exclude_chat_types, includeChatIds=include_chat_ids, includeChatTypes=include_chat_types)
    
    @async_to_sync
    async def get_folders(self, last_state:Optional[str] = None) -> dict:
        return await self.methods.getFolders(lastState=last_state)
    
    @async_to_sync
    async def get_suggested_folders(self) -> dict:
        return await self.methods.getSuggestedFolders()
    
    @async_to_sync
    async def delete_folder(self, folder_id:str) -> dict:
        return await self.methods.deleteFolder(folderId=folder_id)
    
    @async_to_sync
    async def update_profile(self, first_name:Optional[str] = None, last_name:Optional[str] = None, bio:Optional[str] = None, username:Optional[str] = None) -> dict:
        return await self.methods.updateProfile(firstName=first_name, lastname=last_name, bio=bio, username=username)
    
    @async_to_sync
    async def get_my_sessions(self) -> dict:
        return await self.methods.getMySessions()
    
    @async_to_sync
    async def terminate_session(self, session_key:str) -> dict:
        return await self.methods.terminateSession(sessionKey=session_key)
    
    @async_to_sync
    async def terminate_other_sessions(self) -> dict:
        return await self.methods.terminateOtherSessions()
    
    @async_to_sync
    async def check_two_step_passcode(self, password:str) -> dict:
        return await self.methods.checkTwoStepPasscode(password=password)
    
    @async_to_sync
    async def setup_two_step_verification(self, password:str, hint:str, recovery_email:str) -> dict:
        return await self.methods.setupTwoStepVerification(password=password, hint=hint, recoveryEmail=recovery_email)
    
    @async_to_sync
    async def request_recovery_email(self, password:str, recovery_email:str) -> dict:
        return await self.methods.requestRecoveryEmail(password=password, recoveryEmail=recovery_email)
    
    @async_to_sync
    async def verify_recovery_email(self, password:str, code:str) -> dict:
        return await self.methods.verifyRecoveryEmail(password=password, code=code)
    
    @async_to_sync
    async def turn_off_two_step(self, password:str) -> dict:
        return await self.methods.turnOffTwoStep(password=password)
    
    @async_to_sync
    async def change_password(self, password:str, new_password:str, new_hint:str) -> dict:
        return await self.methods.changePassword(password=password, newPassword=new_password, newHint=new_hint)
    
    @async_to_sync
    async def get_two_passcode_status(self) -> dict:
        return await self.methods.getTwoPasscodeStatus()
    
    @async_to_sync
    async def get_privacy_setting(self) -> dict:
        return await self.methods.getPrivacySetting()
    
    @async_to_sync
    async def get_blocked_users(self, start_id:Optional[str] = None) -> dict:
        return await self.methods.getBlockedUsers(startId=start_id)
    
    # Other methods

    @async_to_sync
    async def get_me(self) -> dict:
        return await self.methods.getMe()
    
    @async_to_sync
    async def transcribe_voice(self, object_guid:str, message_id:str) -> dict:
        return await self.methods.transcribeVoice(objectGuid=object_guid, messageId=message_id)
    
    @async_to_sync
    async def reset_contacts(self) -> dict:
        return await self.methods.resetContacts()
    
    @async_to_sync
    async def get_time(self) -> dict:
        return await self.methods.getTime()
    
    @async_to_sync
    async def get_abs_objects(self, object_guids:list) -> dict:
        return await self.methods.getAbsObjects(objectGuids=object_guids)
    
    @async_to_sync
    async def get_link_from_app_url(self, url:str) -> dict:
        return await self.methods.getLinkFromAppUrl(url=url)
    
    @async_to_sync
    async def search_global_objects(self, search_text:str, filters:Optional[list]=None) -> dict:
        return await self.methods.searchGlobalObjects(searchText=search_text, filters=filters)
    
    @async_to_sync
    async def search_global_messages(self, search_text:str) -> dict:
        return await self.methods.searchGlobalMessages(searchText=search_text)
    
    @async_to_sync
    async def check_join(self, object_guid:str, user_guid:str) -> Optional[bool]:
        return await self.methods.checkJoin(objectGuid=object_guid, userGuid=user_guid)
    
    @async_to_sync
    async def get_profile_link_items(self, object_guid:str) -> dict:
        return await self.methods.getProfileLinkItems(objectGuid=object_guid)
    
    @async_to_sync
    async def get_download_link(self, object_guid:Optional[str] = None, message_id:Optional[str] = None, file_inline:Optional[dict]=None) -> Optional[str]:
        return await self.methods.getDownloadLink(objectGuid=object_guid, messageId=message_id, fileInline=file_inline)
    
    @async_to_sync
    async def download(self, object_guid:Optional[str] = None, message_id:Optional[str] = None, save:bool=False, save_as:Optional[str] = None, file_inline:Optional[dict]=None) -> Optional[dict]:
        return await self.methods.download(objectGuid=object_guid, messageId=message_id, save=save, saveAs=save_as, fileInline=file_inline)
    
    @async_to_sync
    async def request(
        self,
        method: str,
        input: dict = {},
        tmp_session: bool = False,
        attempt: int = 0,
        max_attempt: int = 2
    ) -> dict:
        return await self.methods.request(method=method,input=input,tmpSession=tmp_session,attempt=attempt,maxAttempt=max_attempt)

    @async_to_sync
    async def play_voice(self, object_guid: str, file:str) -> None:
        async def main():
            await self.methods.playVoice(objectGuid=object_guid, file=file)
        await main()
    def on_message(self, filters: Union[List[Filter], List[str], Filter, None]):
        def handler(func):
            self.methods.add_handler(
                func=func,
                filters=filters
            )
        return handler
    
    def run(self) -> None:
        self.methods.run()


from typing import List, Dict, Iterator
import json

class KeyPad:
    def __init__(self):
        self.list_KeyPads: List[Dict] = []

    def _create_button(self, id: str, button_text: str, type: str = "Simple") -> dict:
        return {"id": id, "type": type, "button_text": button_text}

    def append(self, *buttons: dict):
        if not buttons:
            raise ValueError("حداقل یک دکمه باید ارسال شود")

        self.list_KeyPads.append({
            "buttons": list(buttons)
        })

    def simple(self, id: str, button_text: str) -> dict:
        return self._create_button(id, button_text)

    def pop(self, index: int = -1):
        self.list_KeyPads.pop(index)

    def clear(self):
        self.list_KeyPads.clear()

    def get(self) -> list:
        return self.list_KeyPads
    
    def get_data_by_id(self, id: str) -> dict:
        """
            find button by id / پیدا کردن دکمه با آیدی

            Docstring for get_data_by_id
            
            :param id: Description
            :type id: str
            :return: Description
            :rtype: dict[Any, Any]
        """
        for buttons in self.list_KeyPads:
            for button in buttons["buttons"]:
                if button["id"] == id:
                    return button
        
        raise IndexError("The Id Not Found !")
    
    def get_all_by_id(self, id: str) -> List[dict]:
        """
        Find all buttons with given id.
        """
        result = []

        for row in self.list_KeyPads:
            for button in row["buttons"]:
                if button["id"] == id:
                    result.append(button)

        return result

    def remove_by_id(self, id: str) -> dict:
        """
        remove button by id / حذف دکمه با آیدی
        """
        for row_index, row in enumerate(self.list_KeyPads):
            buttons = row["buttons"]

            for btn_index, button in enumerate(buttons):
                if button["id"] == id:
                    removed = buttons.pop(btn_index)

                    # اگر سطر خالی شد، کل سطر حذف شود
                    if not buttons:
                        self.list_KeyPads.pop(row_index)

                    return removed

        raise KeyError(f"Button with id '{id}' not found")

    def remove_all_by_id(self, id: str) -> List[dict]:
        """
        Remove all buttons with given id.
        """
        removed = []

        for row in list(self.list_KeyPads):  # کپی برای حذف امن
            buttons = row["buttons"]

            for button in buttons[:]:
                if button["id"] == id:
                    buttons.remove(button)
                    removed.append(button)

            if not buttons:
                self.list_KeyPads.remove(row)

        if not removed:
            raise KeyError(f"Button with id '{id}' not found")

        return removed

    def get_all_buttons(self) -> List[dict]:
        return [
            button
            for row in self.list_KeyPads
            for button in row["buttons"]
        ]


    def __str__(self) -> str:
        return json.dumps(self.list_KeyPads,indent=4,ensure_ascii=False)

    def __repr__(self) -> str:
        return self.__str__()
    
    def __getitem__(self, index):
        return self.list_KeyPads[index]

    def __iter__(self) -> Iterator[Dict]:
        return iter(self.list_KeyPads)

    def __len__(self) -> int:
        return len(self.list_KeyPads)


from fast_rub import Client
from fast_rub.button import KeyPad
import asyncio

bot = Client("test")

chat_id = "b..."
text = "test"

async def test():
    buttons = KeyPad()
    buttons.add_1row().simple("100", "buttun 1")
    buttons.add_4row().simple("101", "buttun 1", "102", "buttun 2", "103", "buttun 3", "104", "buttun 4")
    sending = await bot.send_message_keypad(chat_id, text, Keypad=buttons.get())
    print(sending)

asyncio.run(test())


import os
from typing import Literal


def get_file_category(filename) -> str:
    _, ext = os.path.splitext(filename)
    ext = ext.lower().lstrip('.')
    if not ext:
        return "unknown"
    file_categories = {
        'image': ['jpg', 'jpeg', 'png', 'gif', 'bmp', 'webp', 'svg', 'ico', 'tiff', 'tif', 'psd'],
        'video': ['mp4', 'avi', 'mkv', 'mov', 'wmv', 'flv', 'webm', 'm4v', 'mpeg', 'mpg', '3gp', 'ts'],
        'audio': ['mp3', 'wav', 'ogg', 'flac', 'aac', 'wma', 'm4a', 'aiff', 'mid', 'midi'],
        'voice': ['amr', '3ga'],
        'document': ['pdf', 'doc', 'docx', 'txt', 'rtf', 'odt', 'xls', 'xlsx', 'ppt', 'pptx', 'csv'],
        'web': ['html', 'htm', 'css', 'js', 'json', 'xml', 'php', 'asp', 'aspx'],
        'code': ['py', 'pyc', 'c', 'cpp', 'cs', 'java', 'rb', 'go', 'rs', 'swift', 'kt'],
        'archive': ['zip', 'rar', '7z', 'tar', 'gz', 'bz2', 'iso'],
        'executable': ['exe', 'dll', 'bat', 'sh']
    }
    for category, extensions in file_categories.items():
        if ext in extensions:
            return category
    return "other"


import logging

LOG_FORMAT = "%(asctime)s | %(levelname)-8s | %(name)s | %(message)s"
DEFAULT_LOG_FILE = "fast_rub.log"

def setup_logging(
    *,
    log_to_file: bool = True,
    log_to_console: bool = True,
    level: int = logging.INFO,
    log_file: str = DEFAULT_LOG_FILE
):
    root_logger = logging.getLogger()
    if root_logger.handlers:
        for h in root_logger.handlers[:]:
            root_logger.removeHandler(h)
    root_logger.setLevel(level)
    formatter = logging.Formatter(LOG_FORMAT)
    if log_to_file:
        fh = logging.FileHandler(log_file, encoding="utf-8")
        fh.setFormatter(formatter)
        root_logger.addHandler(fh)
    if log_to_console:
        sh = logging.StreamHandler()
        sh.setFormatter(formatter)
        root_logger.addHandler(sh)
    return logging.getLogger("fast_rub")

_default_logger = setup_logging(log_to_file=True, log_to_console=True)
logger = logging.getLogger("fast_rub")


from random import (
    choices,
    randint
)
from time import time
from re import (
    finditer,
    sub,
    DOTALL
)
from base64 import b64encode
from io import BytesIO
from tempfile import NamedTemporaryFile
from mutagen import (
    mp3,
    File
)
from filetype import guess
from os import (
    system,
    chmod,
    remove
)
from .configs import Configs
from .reacrions import REACTION_MAP
from typing import Optional



class Utils:
    @staticmethod
    def randomTmpSession() -> str:
        return "".join(choices("abcdefghijklmnopqrstuvwxyz", k=32))

    @staticmethod
    def randomDeviceHash() -> str:
        return "".join(choices("0123456789", k=26))

    @staticmethod
    def randomRnd() -> str:
        return str(randint(-99999999, 99999999))

    @staticmethod
    def privateParse(private: str) -> Optional[str]:
        if private:
            if not private.startswith("-----BEGIN RSA PRIVATE KEY-----"):
                private = "-----BEGIN RSA PRIVATE KEY-----\n" + private

            if not private.endswith("-----END RSA PRIVATE KEY-----"):
                private += "\n-----END RSA PRIVATE KEY-----"

            return private.replace("\n", "\n").strip()

    @staticmethod
    def getState() -> int:
        return int(time()) - 150

    @staticmethod
    def phoneNumberParse(phoneNumber: str) -> str:
        if str(phoneNumber).startswith("0"):
            phoneNumber = phoneNumber[1:]
        elif str(phoneNumber).startswith("98"):
            phoneNumber = phoneNumber[2:]
        elif str(phoneNumber).startswith("+98"):
            phoneNumber = phoneNumber[3:]
        return phoneNumber

    @staticmethod
    def getChatTypeByGuid(objectGuid: str) -> str:
        for chatType in [("u0", "User"), ("g0", "Group"), ("c0", "Channel"), ("s0", "Service"), ("b0", "Bot")]:
            if objectGuid.startswith(chatType[0]):
                return chatType[1]
        return "Unknown"

    @staticmethod
    def isChatType(chatType: str) -> Optional[bool]:
        chatType = chatType.lower()
        for type in ["user", "group", "channel", "service", "bot"]:
            if type == chatType:
                return True

    @staticmethod
    def isMessageType(messageType: str) -> Optional[bool]:
        messageType = messageType.lower()
        for type in ["text", "image", "video", "gif", "video message", "voice", "music", "file"]:
            if type == messageType:
                return True
    
    @staticmethod
    def format_file(type_file: Optional[str] = None) -> Optional[str]:
        if not type_file:
            return None
        for type_,pass_ in {"File":"", "Image":".png", "Voice":".mp3", "Music":".mp3", "Gif":".mp4" , "Video":".mp4"}.items():
            if type_ == type_file:
                name_file = type_+pass_
                return name_file
        return None

    @staticmethod
    def getChatTypeByLink(link: str) -> Optional[str]:
        if "rubika.ir/joing" in link:
            return "Group"
        elif "rubika.ir/joinc" in link:
            return "Channel"

    @staticmethod
    def checkMetadata(text) -> tuple:
        if text is None:
            return [], ""

        real_text = sub(r"``|\*\*|__|~~|--|@@|##|", "", text)
        metadata = []
        conflict = 0
        mentionObjectIndex = 0
        result = []

        patterns = {
            "Mono": r"\`\`([^``]*)\`\`",
            "Bold": r"\*\*([^**]*)\*\*",
            "Italic": r"\_\_([^__]*)\_\_",
            "Strike": r"\~\~([^~~]*)\~\~",
            "Underline": r"\-\-([^__]*)\-\-",
            "Mention": r"\@\@([^@@]*)\@\@",
            "Spoiler": r"\#\#([^##]*)\#\#",
        }

        for style, pattern in patterns.items():
            for match in finditer(pattern, text):
                metadata.append((match.start(), len(match.group(1)), style))

        metadata.sort()

        for start, length, style in metadata:
            if not style == "Mention":
                result.append({
                    "type": style,
                    "from_index": start - conflict,
                    "length": length,
                })
                conflict += 4
            else:
                mentionObjects = [i.group(1) for i in finditer(r"\@\(([^(]*)\)", text)]
                mentionType = Utils.getChatTypeByGuid(objectGuid=mentionObjects[mentionObjectIndex]) or "Link"

                if mentionType == "Link":
                    result.append(
                        {
                            "from_index": start - conflict,
                            "length": length,
                            "link": {
                                "hyperlink_data": {
                                    "url": mentionObjects[mentionObjectIndex]
                                },
                                "type": "hyperlink",
                            },
                            "type": mentionType,
                        }
                    )
                else:
                    result.append(
                        {
                            "type": "MentionText",
                            "from_index": start - conflict,
                            "length": length,
                            "mention_text_object_guid": mentionObjects[mentionObjectIndex],
                            "mention_text_object_type": mentionType
                        }
                    )
                real_text = real_text.replace(f"({mentionObjects[mentionObjectIndex]})", "")
                conflict += 6 + len(mentionObjects[mentionObjectIndex])
                mentionObjectIndex += 1

        return result, real_text

    @staticmethod
    def checkMarkdown(text: str) -> tuple:
        """
        Parse basic Markdown styles and return metadata with plain text.
        Supports: **bold**, *italic*, __underline__, ~~strike~~, `mono`, ||spoiler||, [link](url)
        """
        if not text:
            return [], ""
        
        patterns = {
            "Pre": (r"```(.*?)```",),
            "Bold": (r"\*\*([^\*]+?)\*\*",),
            "Underline": (r"__([^_]+?)__",),
            "Strike": (r"~~([^~]+?)~~",),
            "Mono": (r"`([^`]+?)`",),
            "Spoiler": (r"\|\|([^|]+?)\|\|",),
            "Italic": (r"(?<!\*)\*([^*\n]+?)\*(?!\*)", r"(?<!_)_([^_\n]+?)_(?!_)"),
            "Link": (r"\[([^\]]+?)\]\(([^\)]+?)\)",),
            "Mention": (r"\@\@([^@@]+?)\@\@",),
        }
        
        all_matches = []
        mention_objects = [i.group(1) for i in finditer(r"\@\(([^(]*)\)", text)]
        mention_object_index = 0
        used_positions = set()

        for style, pattern_list in patterns.items():
            for pattern in pattern_list:
                for match in finditer(pattern, text, flags=DOTALL):
                    start, end = match.start(), match.end()
                    
                    if any(pos in used_positions for pos in range(start, end)):
                        continue
                    
                    if style == "Link":
                        link_text, link_url = match.groups()
                        all_matches.append({
                            "start": start,
                            "end": end,
                            "length": len(link_text),
                            "style": style,
                            "content": link_text,
                            "full_match": match.group(0),
                            "extra": link_url
                        })
                    elif style == "Pre":
                        content = match.group(1).strip() if match.group(1) else ""
                        all_matches.append({
                            "start": start,
                            "end": end,
                            "length": len(content),
                            "style": style,
                            "content": content,
                            "full_match": match.group(0),
                            "extra": None
                        })
                    else:
                        content = match.group(1)
                        all_matches.append({
                            "start": start,
                            "end": end,
                            "length": len(content),
                            "style": style,
                            "content": content,
                            "full_match": match.group(0),
                            "extra": None
                        })
                    
                    used_positions.update(range(start, end))
        all_matches.sort(key=lambda x: x["start"])
        metadata = []
        conflict = 0
        real_text_final = text

        for match in all_matches:
            current_index = match["start"] - conflict
            
            if match["style"] == "Link":
                metadata.append({
                    "type": "Link",
                    "from_index": current_index,
                    "length": match["length"],
                    "link_url": match["extra"]
                })
            elif match["style"] == "Mention":
                if mention_object_index < len(mention_objects):
                    object_id = mention_objects[mention_object_index]
                    metadata.append({
                        "type": "MentionText",
                        "from_index": current_index,
                        "length": match["length"],
                        "mention_text_object_guid": object_id,
                        "mention_text_user_id": object_id,
                        "mention_text_object_type": "group"
                    })
                    real_text_final = real_text_final.replace(f"({object_id})", "", 1)
                    conflict += 6 + len(object_id)
                    mention_object_index += 1
            else:
                if current_index >= 0 and current_index + match["length"] <= len(real_text_final):
                    metadata.append({
                        "type": match["style"],
                        "from_index": current_index,
                        "length": match["length"]
                    })

            conflict += len(match["full_match"]) - len(match["content"])
            real_text_final = real_text_final.replace(match["full_match"], match["content"], 1)

        return metadata, real_text_final

    @staticmethod
    def checkHTML(text) -> tuple:
        """
        Parse basic HTML tags and return metadata with plain text
        Supports: <b>, <strong>, <i>, <em>, <code>, <s>, <del>, <a href>, <u>, <span class="tg-spoiler">, <pre>
        Also supports mentions with: <a href="rubika://@username"> or objectId mentions
        """
        if text is None:
            return [], ""
        
        metadata = []
        conflict = 0
        result = []
        patterns = {
            "Pre": (r"<pre>([^<]+?)</pre>",),
            "Bold": (r"<b>([^<]+?)</b>", r"<strong>([^<]+?)</strong>"),
            "Italic": (r"<i>([^<]+?)</i>", r"<em>([^<]+?)</em>"),
            "Code": (r"<code>([^<]+?)</code>",),
            "Strike": (r"<s>([^<]+?)</s>", r"<del>([^<]+?)</del>"),
            "Underline": (r"<u>([^<]+?)</u>",),
            "Spoiler": (r'<span class="tg-spoiler">([^<]+?)</span>',),
            "Link": (r'<a href="([^"]+?)">([^<]+?)</a>',),
            "Mention": (r'<mention>([^<]+?)</mention>',),
        }
        
        mention_objects = [i.group(1) for i in finditer(r'<mention objectId="([^"]+)">([^<]+?)</mention>', text)]
        mention_object_index = 0
        
        all_matches = []
        used_positions = set()

        for style, pattern_list in patterns.items():
            for pattern in pattern_list:
                for match in finditer(pattern, text, flags=DOTALL):
                    start, end = match.start(), match.end()
                    
                    if any(pos in used_positions for pos in range(start, end)):
                        continue
                    
                    if style == "Link":
                        link_url, link_text = match.groups()
                        if link_url.startswith('rubika://'):
                            all_matches.append({
                                "start": start,
                                "end": end,
                                "length": len(link_text),
                                "style": "Mention",
                                "content": link_text,
                                "full_match": match.group(0),
                                "extra": link_url
                            })
                        else:
                            all_matches.append({
                                "start": start,
                                "end": end,
                                "length": len(link_text),
                                "style": style,
                                "content": link_text,
                                "full_match": match.group(0),
                                "extra": link_url
                            })
                    elif style == "Mention":
                        content = match.group(1)
                        all_matches.append({
                            "start": start,
                            "end": end,
                            "length": len(content),
                            "style": style,
                            "content": content,
                            "full_match": match.group(0),
                            "extra": None
                        })
                    elif style == "Pre":
                        content = match.group(1).strip() if match.group(1) else ""
                        all_matches.append({
                            "start": start,
                            "end": end,
                            "length": len(content),
                            "style": style,
                            "content": content,
                            "full_match": match.group(0),
                            "extra": None
                        })
                    else:
                        content = match.group(1)
                        all_matches.append({
                            "start": start,
                            "end": end,
                            "length": len(content),
                            "style": style,
                            "content": content,
                            "full_match": match.group(0),
                            "extra": None
                        })
                    
                    used_positions.update(range(start, end))

        all_matches.sort(key=lambda x: x["start"])
        real_text_final = text

        for match in all_matches:
            current_index = match["start"] - conflict
            if match["style"] == "Link":
                result.append({
                    "type": "Link",
                    "from_index": current_index,
                    "length": match["length"],
                    "link_url": match["extra"]
                })
            elif match["style"] == "Mention":
                if match.get("extra") and match["extra"].startswith('rubika://'):
                    result.append({
                        "type": "MentionText",
                        "from_index": current_index,
                        "length": match["length"],
                        "mention_text_object_guid": match["extra"].replace('rubika://', ''),
                        "mention_text_user_id": match["extra"].replace('rubika://', ''),
                        "mention_text_object_type": "user"
                    })
                elif mention_object_index < len(mention_objects):
                    object_id = mention_objects[mention_object_index]
                    result.append({
                        "type": "MentionText",
                        "from_index": current_index,
                        "length": match["length"],
                        "mention_text_object_guid": object_id,
                        "mention_text_user_id": object_id,
                        "mention_text_object_type": "group"
                    })
                    mention_object_index += 1
            else:
                if current_index >= 0 and current_index + match["length"] <= len(real_text_final):
                    result.append({
                        "type": match["style"],
                        "from_index": current_index,
                        "length": match["length"],
                    })

            conflict += len(match["full_match"]) - len(match["content"])
            real_text_final = real_text_final.replace(match["full_match"], match["content"], 1)

        return result, real_text_final

    @staticmethod
    def checkLink(url: str) -> Optional[bool]:
        for i in ["http:/", "https://"]:
            if url.startswith(i):
                return True

    @staticmethod
    def getMimeFromByte(data: bytes) -> str:
        mime = guess(data)
        return "pyrubi" if mime is None else mime.extension

    @staticmethod
    def generateFileName(mime: str) -> str:
        return "Pyrubi Library {}.{}".format(randint(1, 1000), mime)

    @staticmethod
    def getImageSize(data: bytes) -> tuple:
        try:
            from PIL import Image
        except ImportError:
            system("pip install pillow")
            from PIL import Image
        width, height = Image.open(BytesIO(data)).size
        return width, height

    @staticmethod
    def getImageThumbnail(data: bytes) -> str:
        try:
            from PIL import Image
        except ImportError:
            system("pip install pillow")
            from PIL import Image
        image = Image.open(BytesIO(data))
        width, height = image.size
        if height > width:
            new_height = 40
            new_width = round(new_height * width / height)
        else:
            new_width = 40
            new_height = round(new_width * height / width)
        image = image.resize((new_width, new_height), Image.LANCZOS)
        changed_image = BytesIO()
        image.save(changed_image, format="PNG")
        return b64encode(changed_image.getvalue()).decode("UTF-8")

    @staticmethod
    def getVideoData(data: bytes) -> tuple:
        try:
            from moviepy.editor import VideoFileClip
            with NamedTemporaryFile(delete=False, dir=".") as temp_video:
                temp_video.write(data)
                temp_path = temp_video.name
            chmod(temp_path, 0o777)
            try:
                from PIL import Image
            except ImportError:
                system("pip install pillow")
                from PIL import Image
            with VideoFileClip(temp_path) as clip:
                duration = clip.duration
                resolution = clip.size
                thumbnail = clip.get_frame(0)
                thumbnail_image = Image.fromarray(thumbnail)
                thumbnail_buffer = BytesIO()
                thumbnail_image.save(thumbnail_buffer, format="JPEG")
                thumbnail_b64 = b64encode(thumbnail_buffer.getvalue()).decode("UTF-8")
                clip.close()
            remove(temp_path)
            return thumbnail_b64, resolution, duration
        except ImportError:
            print("Can't get video data! Please install the moviepy library with: pip install moviepy")
            return Configs.defaultTumbInline, [900, 720], 1
        except:
            return Configs.defaultTumbInline, [900, 720], 1

    @staticmethod
    def getVoiceDuration(data: bytes) -> int:
        file = BytesIO()
        file.write(data)
        file.seek(0)
        return mp3.MP3(file).info.length

    @staticmethod
    def getMusicArtist(data: bytes) -> str:
        try:
            audio = File(BytesIO(data), easy=True)
            if audio and "artist" in audio:
                return audio["artist"][0]
            return "pyrubi"
        except Exception:
            return "pyrubi"
    
    @staticmethod
    def reaction_to_id(reaction: str) -> int:
        if len(reaction) != 1:
            raise ValueError("The reaction not has 1 len !")
        if not reaction in REACTION_MAP.keys():
            raise ValueError("The reaction is not valid !")
        return REACTION_MAP[reaction]


class Colors:
    RESET = "\033[0m"
    BLACK = "\033[30m"
    RED = "\033[31m"
    GREEN = "\033[32m"
    YELLOW = "\033[33m"
    BLUE = "\033[34m"
    MAGENTA = "\033[35m"
    CYAN = "\033[36m"
    WHITE = "\033[37m"
    BG_RESET = "\033[49m"
    BG_BLACK = "\033[40m"
    BG_RED = "\033[41m"
    BG_GREEN = "\033[42m"
    BG_YELLOW = "\033[43m"
    BG_BLUE = "\033[44m"
    BG_MAGENTA = "\033[45m"
    BG_CYAN = "\033[46m"
    BG_WHITE = "\033[47m"



from setuptools import setup, find_packages

setup(
    name="fastrub",
    version="2.5.0",
    author="seyyed mohamad hosein moosavi raja(01)",
    author_email="mohamadhosein159159@gmail.com",
    description="the library for rubika bots.",
    long_description=open("README.md",encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/OandONE/fast_rub",
    packages=find_packages(),
    python_requires='>=3.9',
    install_requires=[
        "httpx==0.28.1", # fast_rub - sending requests
        "httpx[http2]", # fast_rub - sending requests
        "aiofiles==24.1.0", # fast_rub - saveing files async
        "filetype", # fork pyrubi
        "mutagen", # fork pyrubi
        "pycryptodome", # fork pyrubi
        "tqdm", # fork pyrubi
        "websocket-client" # fork pyrubi
    ],
    license="MIT"
)


import httpx
import asyncio
from typing import Optional, Union, Dict, Any, List, Literal
from ..type.errors import ServerRubikaError
import os
import aiofiles

class Network:
    def __init__(
        self,
        token: str,
        logger,
        max_retries: int = 3,
        user_agent: Optional[str] = None,
        main_url: str = "https://botapi.rubika.ir/v3/",
        proxy: Optional[str] = None,
        rate_limit: int = 20
    ):
        self.logger = logger
        self.token = token
        self.user_agent = user_agent
        self.main_url = main_url
        self.proxy = proxy
        self.max_retries = max_retries
        self._client: Optional[httpx.AsyncClient] = None
        self._client_lock = asyncio.Lock()
        self._rate_sem = asyncio.Semaphore(rate_limit)
        self._worker_task: Optional[asyncio.Task] = None
        self._queue: asyncio.Queue = asyncio.Queue()
        self._closed = False

    # -------------------
    # Client management
    # -------------------
    async def _create_client(self):
        self._client = httpx.AsyncClient(
            timeout=httpx.Timeout(connect=10.0, read=30.0, write=10.0, pool=10.0),
            proxy=self.proxy,
            limits=httpx.Limits(max_connections=100, max_keepalive_connections=20),
            http1=True,
            http2=True,
            headers={
                "Content-Type": "application/json",
                "User-Agent": self.user_agent or "fast_rub/Network"
            }
        )
        self.logger.debug("HTTP client created")

    async def _ensure_client(self):
        if self._client is None or self._client.is_closed:
            async with self._client_lock:
                if self._client is None or self._client.is_closed:
                    await self._create_client()

    async def close(self):
        self._closed = True
        if self._worker_task:
            self._worker_task.cancel()
            try:
                await self._worker_task
            except asyncio.CancelledError:
                pass
        if self._client and not self._client.is_closed:
            await self._client.aclose()
        self.logger.debug("Network closed")

    # -------------------
    # Worker queue
    # -------------------
    async def start_worker(self):
        if self._worker_task is None:
            self._worker_task = asyncio.create_task(self._worker())
            self.logger.debug("Worker started")

    async def _worker(self):
        while not self._closed:
            try:
                url, method, data_, headers, overrides, fut = await self._queue.get()
            except asyncio.CancelledError:
                break
            try:
                res = await self._do_request(url, method, data_, headers, **overrides)
                if not fut.done():
                    fut.set_result(res)
            except Exception as e:
                if not fut.done():
                    fut.set_exception(e)
            finally:
                self._queue.task_done()

    # -------------------
    # Public request
    # -------------------
    async def request(
        self,
        url: str,
        data_: Optional[Union[Dict[str, Any], List[Any]]] = None,
        type_send: Literal["POST", "GET"] = "POST",
        *,
        max_retries: Optional[int] = None,
        timeout: Optional[float] = None
    ) -> httpx.Response:
        if self._closed:
            raise RuntimeError("Network is closed")

        overrides = {
            "max_retries": max_retries or self.max_retries,
            "timeout": timeout or 30.0
        }

        # مستقیم درخواست را بفرست، بدون Queue
        return await self._do_request(url, type_send, data_, None, **overrides)

    # -------------------
    # Internal request with retry
    # -------------------
    async def _do_request(
        self,
        url: str,
        method: str,
        data_: Optional[Union[Dict, List]],
        headers: Optional[Dict[str, str]],
        *,
        max_retries: int,
        timeout: float
    ) -> httpx.Response:

        last_exception = None
        headers = headers or {"Content-Type": "application/json"}
        if self.user_agent:
            headers["User-Agent"] = self.user_agent

        # ابتدا Client را فقط یک بار آماده می‌کنیم
        await self._ensure_client()

        for attempt in range(1, max_retries + 1):
            try:
                # Semaphore را فقط هنگام ارسال درخواست بگیریم
                async with self._rate_sem:
                    if method == "POST":
                        resp = await self._client.post(url, json=data_, headers=headers, timeout=timeout)
                    elif method == "GET":
                        resp = await self._client.get(url, headers=headers, timeout=timeout)
                    else:
                        raise ValueError(f"Invalid method: {method}")

                resp.raise_for_status()
                return resp

            except (httpx.ReadError, httpx.ConnectError, httpx.TimeoutException) as e:
                last_exception = e
                wait_time = attempt * 1.5
                self.logger.warning(f"Request failed attempt {attempt}/{max_retries}: {e}. Retrying in {wait_time}s...")

                # فقط Client را ببند و دوباره بساز، بدون نگه داشتن Lock طولانی
                async with self._client_lock:
                    if self._client:
                        try:
                            await self._client.aclose()
                        except:
                            pass
                        self._client = None

                # دوباره Client را آماده کن قبل از Retry بعدی
                await self._ensure_client()
                await asyncio.sleep(wait_time)

            except Exception as e:
                self.logger.error(f"Unexpected error during request: {e}")
                raise e

        self.logger.error(f"All {max_retries} attempts failed for {url}")
        raise last_exception or RuntimeError("Unknown error in request")

    # -------------------
    # Rubika high-level
    # -------------------
    async def send_request(
        self,
        method: str,
        data_: Optional[Union[Dict[str, Any], List[Any]]] = None
    ) -> dict:
        self.logger.debug(f"method {method}")
        url = f"{self.main_url}{self.token}/{method}"
        resp = await self.request(url, data_, "POST")
        try:
            result = resp.json()
        except Exception:
            raise ServerRubikaError("Error converting response to JSON")

        if result.get("status", "") != "OK":
            self.logger.error(f"Server returned error: {result}")
            raise ServerRubikaError(result)
        return result
    
    async def download(self, url: str, path: str = "file") -> bool:
        try:
            await self._ensure_client()  # مهم: تضمین ساخته شدن Client

            dir_path = os.path.dirname(path)
            if dir_path:
                os.makedirs(dir_path, exist_ok=True)

            async with self._rate_sem:  # رعایت Rate Limit
                async with self._client.stream("GET", url) as response:
                    response.raise_for_status()
                    async with aiofiles.open(path, 'wb') as file:
                        async for chunk in response.aiter_bytes():
                            await file.write(chunk)
            return True
        except Exception as e:
            self.logger.error(f"Download failed: {e}")
            return False



class Encryption:
    @staticmethod
    def _clip(n):
        return max(32, min(126, n))
    @staticmethod
    def _compute_enc(code, pos, pat):
        if pat == 0:
            return code + pos * 10, "A"
        elif pat == 1:
            return code * 3 + pos, "B"
        elif pat == 2:
            return (code << 2) + pos * 5, "C"
        else:
            return 1000 - code + pos * 7, "D"
    @staticmethod
    def _compute_dec(val, pos, mark):
        if mark == "A":
            return val - pos * 10
        elif mark == "B":
            return (val - pos) // 3
        elif mark == "C":
            return (val - pos * 5) >> 2
        else:
            return 1000 - val + pos * 7
    @staticmethod
    def _scramble_hex(h):
        return h[::-1]
    @staticmethod
    def _unscramble_hex(h):
        return h[::-1]
    @staticmethod
    def en(text):
        out = []
        for i, ch in enumerate(text):
            pos = i + 1
            pat = i % 4
            code = ord(ch)
            val, mark = Encryption._compute_enc(code, pos, pat)
            hex_val = f"{val:06X}"
            scr = Encryption._scramble_hex(hex_val)
            block = f"{mark}{pos:03d}{scr}"
            out.append(block)
        return "{" + ",".join(out) + "}"
    @staticmethod
    def de(encoded):
        encoded = encoded.strip("{}")
        blocks = encoded.split(",")
        out = []
        for block in blocks:
            mark = block[0]
            pos = int(block[1:4])
            scr_hex = block[4:]
            hex_val = Encryption._unscramble_hex(scr_hex)
            val = int(hex_val, 16)
            code = Encryption._compute_dec(val, pos, mark)
            out.append(chr(Encryption._clip(code)))
        return "".join(out)


import re
from typing import List, Dict, Any, Tuple, Optional
from .colors import cprint, Colors
import time
from .encryption import Encryption
import json

def _collect_matches(text: str, patterns: Dict[str, List[str]], priority: Dict[str, int]) -> List[Dict[str, Any]]:
    matches = []
    for style, pats in patterns.items():
        for pat in pats:
            for m in re.finditer(pat, text, flags=re.DOTALL | re.MULTILINE):
                start, end = m.start(), m.end()
                groups = m.groups()
                content = ""
                extra = None

                if style == "Link" and len(groups) >= 2:
                    content = groups[0]
                    extra = groups[1]
                elif style == "HTMLLink" and len(groups) >= 2:
                    extra = groups[0]
                    content = groups[1]
                elif style == "MentionHTML" and len(groups) >= 2:
                    extra = groups[0]
                    content = groups[1]
                else:
                    if len(groups) >= 1:
                        content = groups[0]
                    else:
                        content = m.group(0)

                if style in ("Pre", "PreHTML"):
                    content = content.strip("\n")

                matches.append({
                    "start": start,
                    "end": end,
                    "style": style,
                    "content": content,
                    "full_match": m.group(0),
                    "extra": extra,
                    "priority": priority.get(style, 50)
                })
    return matches

def _allow_match(chosen: List[Dict[str, Any]], candidate: Dict[str, Any]) -> bool:
    s, e = candidate["start"], candidate["end"]
    for c in chosen:
        os, oe = c["start"], c["end"]
        if (s >= os and e <= oe) or (os >= s and oe <= e):
            continue
        if not (e <= os or s >= oe):
            return False
    return True

def _pick_matches_allowing_nested(matches: List[Dict[str, Any]]) -> List[Dict[str, Any]]:
    matches_sorted = sorted(matches, key=lambda m: (m['priority'], m['start']))
    chosen = []
    for m in matches_sorted:
        if _allow_match(chosen, m):
            chosen.append(m)
    chosen.sort(key=lambda m: m['start'])
    return chosen

class TextParser:
    @staticmethod
    def checkMarkdown(text: str) -> Tuple[List[Dict[str, Any]], str]:
        """
        Parse Markdown and return flat metadata list + plain text.
        - Nested styles are preserved as separate metadata entries (flat list).
        - Partial/cross overlaps are rejected.
        """
        if not text:
            return [], ""

        patterns = {
            "Pre": [r"```(?:[^\n]*\n)?([\s\S]*?)```"],
            "Link": [r"\[([^\]]+?)\]\(([^)]+?)\)"],  # [text](url)
            "MentionMD": [r"@@([^@]+?)@@"],
            "CodeInline": [r"`([^`]+?)`"],
            "Spoiler": [r"\|\|([^|]+?)\|\|"],
            "Bold": [r"\*\*([^\*]+?)\*\*", r"__(?<!_)([^_]+?)__(?!_)"],
            "Strike": [r"~~([^~]+?)~~"],
            "Underline": [r"__(?<!_)([^_]+?)__(?!_)"],
            "Italic": [r"(?<!\*)\*([^*\n]+?)\*(?!\*)", r"(?<!_)_([^_\n]+?)_(?!_)"],
            "Blockquote": [r"(^> .+(?:\n> .+)*)"]
        }

        priority = {
            "Pre": 0,
            "Link": 1,
            "MentionMD": 1,
            "CodeInline": 2,
            "Spoiler": 3,
            "Bold": 4,
            "Strike": 5,
            "Underline": 6,
            "Italic": 7,
            "Blockquote": 8
        }

        all_matches = _collect_matches(text, patterns, priority)
        chosen = _pick_matches_allowing_nested(all_matches)

        out_parts: List[str] = []
        metadata: List[Dict[str, Any]] = []
        last = 0

        for m in chosen:
            if last < m['start']:
                out_parts.append(text[last:m['start']])
            current_index = sum(len(p) for p in out_parts)
            out_parts.append(m['content'])
            length = len(m['content'])
            st = m['style']

            if st == "Link":
                metadata.append({
                    "type": "Link",
                    "from_index": current_index,
                    "length": length,
                    "link_url": m['extra']
                })
            elif st == "MentionMD":
                metadata.append({
                    "type": "MentionText",
                    "from_index": current_index,
                    "length": length,
                    "mention_text_object_guid": m['content'],
                    "mention_text_user_id": m['content'],
                    "mention_text_object_type": "user"
                })
            elif st == "CodeInline":
                metadata.append({
                    "type": "Mono",
                    "from_index": current_index,
                    "length": length
                })
            elif st == "Pre":
                metadata.append({
                    "type": "Pre",
                    "from_index": current_index,
                    "length": length
                })
            elif st == "Blockquote":
                lines = re.sub(r"^> ?", "", m['content'], flags=re.MULTILINE)
                metadata.append({
                    "type": "Blockquote",
                    "from_index": current_index,
                    "length": length
                })
            else:
                map_types = {
                    "Bold": "Bold",
                    "Italic": "Italic",
                    "Underline": "Underline",
                    "Strike": "Strike",
                    "Spoiler": "Spoiler"
                }
                meta_type = map_types.get(st, st)
                metadata.append({
                    "type": meta_type,
                    "from_index": current_index,
                    "length": length
                })

            last = m['end']

        if last < len(text):
            out_parts.append(text[last:])

        real_text_final = "".join(out_parts)
        return metadata, real_text_final

    @staticmethod
    def checkHTML(text: str) -> Tuple[List[Dict[str, Any]], str]:
        """
        Parse simple HTML-like tags and return flat metadata + plain text.
        Supports <b>, <strong>, <i>, <em>, <code>, <s>, <del>, <u>, <pre>, <span class="tg-spoiler">,
        <a href="...">text</a>, <mention objectId="...">text</mention>, and rubika:// links.
        """
        if text is None:
            return [], ""

        patterns = {
            "PreHTML": [r"<pre>([\s\S]*?)</pre>"],
            "HTMLLink": [r'<a\s+href="([^"]+?)">([^<]+?)</a>'],
            "MentionHTML": [r'<mention\s+objectId="([^"]+?)">([^<]+?)</mention>'],
            "CodeInlineHTML": [r"<code>([^<]+?)</code>"],
            "SpoilerHTML": [r'<span\s+class="tg-spoiler">([^<]+?)</span>'],
            "BoldHTML": [r"<b>([^<]+?)</b>", r"<strong>([^<]+?)</strong>"],
            "ItalicHTML": [r"<i>([^<]+?)</i>", r"<em>([^<]+?)</em>"],
            "StrikeHTML": [r"<s>([^<]+?)</s>", r"<del>([^<]+?)</del>"],
            "UnderlineHTML": [r"<u>([^<]+?)</u>"],
        }

        priority = {
            "PreHTML": 0,
            "HTMLLink": 1,
            "MentionHTML": 1,
            "CodeInlineHTML": 2,
            "SpoilerHTML": 3,
            "BoldHTML": 4,
            "ItalicHTML": 5,
            "StrikeHTML": 6,
            "UnderlineHTML": 7
        }

        all_matches = _collect_matches(text, patterns, priority)
        chosen = _pick_matches_allowing_nested(all_matches)

        out_parts: List[str] = []
        metadata: List[Dict[str, Any]] = []
        last = 0

        for m in chosen:
            if last < m['start']:
                out_parts.append(text[last:m['start']])
            current_index = sum(len(p) for p in out_parts)
            out_parts.append(m['content'])
            length = len(m['content'])
            st = m['style']

            if st == "HTMLLink":
                url = m['extra']
                if url and url.startswith("rubika://"):
                    uid = url.replace("rubika://", "")
                    metadata.append({
                        "type": "MentionText",
                        "from_index": current_index,
                        "length": length,
                        "mention_text_object_guid": uid,
                        "mention_text_user_id": uid,
                        "mention_text_object_type": "user"
                    })
                else:
                    metadata.append({
                        "type": "Link",
                        "from_index": current_index,
                        "length": length,
                        "link_url": url
                    })
            elif st == "MentionHTML":
                object_id = m['extra'] or m['content']
                metadata.append({
                    "type": "MentionText",
                    "from_index": current_index,
                    "length": length,
                    "mention_text_object_guid": object_id,
                    "mention_text_user_id": object_id,
                    "mention_text_object_type": "group"
                })
            elif st == "CodeInlineHTML":
                metadata.append({
                    "type": "Mono",
                    "from_index": current_index,
                    "length": length
                })
            elif st == "PreHTML":
                metadata.append({
                    "type": "Pre",
                    "from_index": current_index,
                    "length": length
                })
            elif st == "SpoilerHTML":
                metadata.append({
                    "type": "Spoiler",
                    "from_index": current_index,
                    "length": length
                })
            else:
                map_html = {
                    "BoldHTML": "Bold",
                    "ItalicHTML": "Italic",
                    "UnderlineHTML": "Underline",
                    "StrikeHTML": "Strike"
                }
                meta_type = map_html.get(st, st)
                metadata.append({
                    "type": meta_type,
                    "from_index": current_index,
                    "length": length
                })

            last = m['end']

        if last < len(text):
            out_parts.append(text[last:])

        real_text_final = "".join(out_parts)
        return metadata, real_text_final

class utils:
    @staticmethod
    def format_file(type_file: Optional[str] = None) -> Optional[str]:
        if not type_file:
            return None
        for type_,pass_ in {"File":"", "Image":".png", "Voice":".mp3", "Music":".mp3", "Gif":".mp4" , "Video":".mp4"}.items():
            if type_ == type_file:
                name_file = type_+pass_
                return name_file
        return None
    
    @staticmethod
    def print_time(text: str, time_sleep: float = 0.07) -> None:
        k = ""
        for text in text:
            k += text
            print(f"{Colors.BLUE}{k}{Colors.RESET}", end="\r")
            time.sleep(time_sleep)
        cprint("",Colors.WHITE)
    
    @staticmethod
    def get_input(text_output: str) -> str:
        text = None
        while text in ["", " ", None] or len(str(text)) != 64:
            cprint("Write the valid ! Your text invalid.",Colors.RED)
            text = input(text_output)
        return text
    
    @staticmethod
    def session_dict(name_session: str, token: str, user_agent: Optional[str] = None, time_out: Optional[int] = 30, display_welcome: bool = False, view_logs: Optional[bool] = False, save_logs: Optional[bool] = False) -> dict:
        text_json_fast_rub_session = {
            "name_session": name_session,
            "token": token,
            "user_agent": user_agent,
            "time_out": time_out,
            "display_welcome": display_welcome,
            "setting_logs":{
                "view":view_logs,
                "save":save_logs
            }
        }
        return text_json_fast_rub_session
    
    @staticmethod
    def create_session(name_session: str, token: Optional[str] = None, user_agent: Optional[str] = None, time_out: Optional[int] = 30, display_welcome: bool = False, view_logs: Optional[bool] = False, save_logs: Optional[bool] = False) -> bool:
        try:
            if token is None:
                token = utils.get_input("Write The Token » ")
            session = utils.session_dict(name_session,token,user_agent,time_out,display_welcome,view_logs,save_logs)
            json_fast_rub_session = json.dumps(session,indent=4,ensure_ascii=False)
            json_fast_rub_session = Encryption().en(str(json_fast_rub_session))
            with open(f"{name_session}.faru", "w", encoding="utf-8") as file:
                file.write(json_fast_rub_session)
            return True
        except:
            return False


import json

class file:
    def __init__(self,data: dict):
        self.data = data
    @property
    def file_id(self) -> str:
        return self.data["file_id"]
    @property
    def file_name(self) -> str:
        return self.data["file_name"]
    @property
    def size(self) -> int:
        return self.data["size"]
    def __str__(self) -> str:
        return json.dumps(self.data,indent=4,ensure_ascii=False)
    def __repr__(self) -> str:
        return self.__str__()


from ..Client import Client
from typing import Literal, Optional
from .Update import Update
import json
from .props import props

class msg_update:
    def __init__(self, update_data: dict, client: "Client") -> None:
        self.update_data = update_data["data"]
        self.client = client
    
    @property
    def message_id(self) -> str:
        "message id / آیدی پیام"
        return self.update_data["message_id"]
    
    @property
    def chat_id(self) -> str:
        "chat id / چت آیدی"
        return self.update_data["chat_id"]

    async def edit_text(self, new_text: str, parse_mode: Literal['Markdown', 'HTML'] | None = "Markdown") -> "msg_update":
        "edit text message / ویرایش متن پیام"
        return await self.client.edit_message_text(
            self.chat_id,
            self.message_id,
            new_text,
            parse_mode
        )
    
    async def get_message(self) -> Optional[Update]:
        """getting message info / گرفتن اطلاعات پیام"""
        return await self.client.get_message(self.chat_id,self.message_id)
    
    async def delete_message(self) -> props:
        """حذف پیام / delete message
        
        Docstring for delete_message
        
        :return: Description
        :rtype: props
        """
        return await self.client.delete_message(self.chat_id, self.message_id)

    def __str__(self) -> str:
        return json.dumps(self.update_data,indent=4,ensure_ascii=False)

    def __repr__(self) -> str:
        return self.__str__()